@@ -22,6 +22,8 @@ def cmd_retrieval_common_opts(sp):
                     help='When accepting messages on stdin, split using this pipe separator string')
     sp.add_argument('-C', '--no-cache', dest='nocache', action='store_true', default=False,
                     help='Do not use local cache')
+    sp.add_argument('--single-message', dest='singlemsg', action='store_true', default=False,
+                    help='Only retrieve the message matching the msgid and ignore the rest of the thread')
 
 
 def cmd_mbox_common_opts(sp):
@@ -40,7 +42,7 @@ def cmd_am_common_opts(sp):
     sp.add_argument('-v', '--use-version', dest='wantver', type=int, default=None,
                     help='Get a specific version of the patch/series')
     sp.add_argument('-t', '--apply-cover-trailers', dest='covertrailers', action='store_true', default=False,
-                    help='Apply trailers sent to the cover letter to all patches')
+                    help='(This is now the default behavior; this option will be removed in the future.)')
     sp.add_argument('-S', '--sloppy-trailers', dest='sloppytrailers', action='store_true', default=False,
                     help='Apply trailers without email address match checking')
     sp.add_argument('-T', '--no-add-trailers', dest='noaddtrailers', action='store_true', default=False,
@@ -111,6 +113,24 @@ def cmd_diff(cmdargs):
     b4.diff.main(cmdargs)
 
 
+class ConfigOption(argparse.Action):
+    """Action class for storing key=value arguments in a dict."""
+    def __call__(self, parser, namespace, keyval, option_string=None):
+        config = getattr(namespace, self.dest, None)
+
+        if config is None:
+            config = dict()
+            setattr(namespace, self.dest, config)
+
+        if '=' in keyval:
+            key, value = keyval.split('=', maxsplit=1)
+        else:
+            # mimic git -c option
+            key, value = keyval, 'true'
+
+        config[key] = value
+
+
 def setup_parser() -> argparse.ArgumentParser:
     # noinspection PyTypeChecker
     parser = argparse.ArgumentParser(
@@ -130,6 +150,14 @@ def setup_parser() -> argparse.ArgumentParser:
                         help='Do not perform any network queries')
     parser.add_argument('--no-stdin', action='store_true', default=False,
                         help='Disable TTY detection for stdin')
+    parser.add_argument('--use-web-endpoint', dest='send_web', action='store_true', default=False,
+                        help="Force going through the web endpoint")
+    parser.add_argument('-c', '--config', metavar='NAME=VALUE', action=ConfigOption,
+                        help='''Set config option NAME to VALUE. Override value
+                        from config files. NAME is in dotted section.key
+                        format. Using NAME= and omitting VALUE will set the
+                        value to the empty string. Using NAME and omitting
+                        =VALUE will set the value to "true".''')
 
     subparsers = parser.add_subparsers(help='sub-command help', dest='subcmd')
 
@@ -175,6 +203,8 @@ def setup_parser() -> argparse.ArgumentParser:
     sp_sh.add_argument('--guess-lookback', dest='guessdays', type=int, default=21,
                        help=('(use with -H or -M) When guessing base, go back this many days from the patch date '
                              '(default: 3 weeks)'))
+    sp_sh.add_argument('--merge-base', dest='mergebase', type=str, default=None,
+                       help='(use with -H or -M) Force this base when merging')
     sp_sh.set_defaults(func=cmd_shazam)
 
     # b4 pr
@@ -189,8 +219,6 @@ def setup_parser() -> argparse.ArgumentParser:
                        help='Convert a pull request into an mbox full of patches')
     sp_pr.add_argument('-o', '--output-mbox', dest='outmbox', default=None,
                        help='Save exploded messages into this mailbox (default: msgid.mbx)')
-    sp_pr.add_argument('-l', '--retrieve-links', action='store_true', dest='getlinks', default=False,
-                       help='Attempt to retrieve any Link: URLs (use with -e)')
     sp_pr.add_argument('-f', '--from-addr', dest='mailfrom', default=None,
                        help='Use this From: in exploded messages (use with -e)')
     sp_pr.add_argument('-s', '--send-as-identity', dest='sendidentity', default=None,
@@ -258,11 +286,11 @@ def setup_parser() -> argparse.ArgumentParser:
     # b4 prep
     sp_prep = subparsers.add_parser('prep', help='Work on patch series to submit for mailing list review')
     sp_prep.add_argument('-c', '--auto-to-cc', action='store_true', default=False,
-                       help='Automatically populate cover letter trailers with To and Cc addresses')
+                         help='Automatically populate cover letter trailers with To and Cc addresses')
     sp_prep.add_argument('--force-revision', metavar='N', type=int,
-                       help='Force revision to be this number instead')
+                         help='Force revision to be this number instead')
     sp_prep.add_argument('--set-prefixes', metavar='PREFIX', nargs='+',
-                       help='Extra prefixes to add to [PATCH] (e.g.: RFC mydrv)')
+                         help='Extra prefixes to add to [PATCH] (e.g.: RFC mydrv)')
 
     spp_g = sp_prep.add_mutually_exclusive_group()
     spp_g.add_argument('-p', '--format-patch', metavar='OUTPUT_DIR',
@@ -275,8 +303,10 @@ def setup_parser() -> argparse.ArgumentParser:
                        help='Display a range-diff to previously sent revision N')
     spp_g.add_argument('--manual-reroll', dest='reroll', default=None, metavar='COVER_MSGID',
                        help='Mark current revision as sent and reroll (requires cover letter msgid)')
-    spp_g.add_argument('--show-info', action='store_true', default=False,
-                       help='Show current series info in a column-parseable format')
+    spp_g.add_argument('--show-info', metavar='PARAM', nargs='?', const=':_all',
+                       help='Show series info in a format that can be passed to other commands.')
+    spp_g.add_argument('--cleanup', metavar='BRANCHNAME', nargs='?', const='_show',
+                       help='Archive and remove a prep-tracked branch and all its sent/ tags')
 
     ag_prepn = sp_prep.add_argument_group('Create new branch', 'Create a new branch for working on patch series')
     ag_prepn.add_argument('-n', '--new', dest='new_series_name',
@@ -286,8 +316,9 @@ def setup_parser() -> argparse.ArgumentParser:
     ag_prepn.add_argument('-F', '--from-thread', metavar='MSGID', dest='msgid',
                           help='When creating a new branch, use this thread')
     ag_prepe = sp_prep.add_argument_group('Enroll existing branch', 'Enroll existing branch for prep work')
-    ag_prepe.add_argument('-e', '--enroll', dest='enroll_base',
-                          help='Enroll current branch, using the passed tag, branch, or commit as fork base')
+    ag_prepe.add_argument('-e', '--enroll', dest='enroll_base', nargs='?', const='@{upstream}',
+                          help='Enroll current branch, using its configured upstream branch as fork base, '
+                               'or the passed tag, branch, or commit')
     sp_prep.set_defaults(func=cmd_prep)
 
     # b4 trailers
@@ -305,16 +336,20 @@ def setup_parser() -> argparse.ArgumentParser:
 
     # b4 send
     sp_send = subparsers.add_parser('send', help='Submit your work for review on the mailing lists')
-    sp_send.add_argument('-d', '--dry-run', dest='dryrun', action='store_true', default=False,
-                         help='Do not send, just dump out raw smtp messages to the stdout')
-    sp_send.add_argument('-o', '--output-dir',
-                         help='Do not send, write raw messages to this directory (forces --dry-run)')
-    sp_send.add_argument('--reflect', action='store_true', default=False,
-                         help='Send everything to yourself instead of the actual recipients')
+    sp_send_g = sp_send.add_mutually_exclusive_group()
+    sp_send_g.add_argument('-d', '--dry-run', dest='dryrun', action='store_true', default=False,
+                           help='Do not send, just dump out raw smtp messages to the stdout')
+    sp_send_g.add_argument('-o', '--output-dir',
+                           help='Do not send, write raw messages to this directory (forces --dry-run)')
+    sp_send_g.add_argument('--preview-to', nargs='+', metavar='ADDR',
+                           help='Send everything for a pre-review to specified addresses instead of actual recipients')
+    sp_send_g.add_argument('--reflect', action='store_true', default=False,
+                           help='Send everything to yourself instead of the actual recipients')
+
     sp_send.add_argument('--no-trailer-to-cc', action='store_true', default=False,
                          help='Do not add any addresses found in the cover or patch trailers to To: or Cc:')
-    sp_send.add_argument('--to', nargs='+', help='Addresses to add to the To: list')
-    sp_send.add_argument('--cc', nargs='+', help='Addresses to add to the Cc: list')
+    sp_send.add_argument('--to', nargs='+', metavar='ADDR', help='Addresses to add to the To: list')
+    sp_send.add_argument('--cc', nargs='+', metavar='ADDR', help='Addresses to add to the Cc: list')
     sp_send.add_argument('--not-me-too', action='store_true', default=False,
                          help='Remove yourself from the To: or Cc: list')
     sp_send.add_argument('--resend', metavar='vN', nargs='?', const='latest',
@@ -353,6 +388,8 @@ def cmd():
         parser.print_help()
         sys.exit(1)
 
+    b4.setup_config(cmdargs)
+
     if cmdargs.offline_mode:
         logger.info('Running in OFFLINE mode')
         b4.can_network = False
@@ -363,6 +400,7 @@ def cmd():
 if __name__ == '__main__':
     # We're running from a checkout, so reflect git commit in the version
     import os
+
     # noinspection PyBroadException
     try:
         if b4.__VERSION__.find('-dev') > 0:
