@@ -87,11 +87,12 @@ def test_parse_trailers(sampledir, source, expected):
     ('single', {}, {'addmysob': True}, 'addmysob', {}),
     ('single', {}, {'addmysob': True, 'copyccs': True}, 'copyccs', {}),
     ('single', {}, {'addmysob': True, 'addlink': True}, 'addlink', {}),
+    ('single', {}, {'addmysob': True, 'addlink': True}, 'addmsgid', {'linktrailermask': 'Message-Id: <%s>'}),
     ('single', {}, {'addmysob': True, 'copyccs': True}, 'ordered',
      {'trailer-order': 'Cc,Tested*,Reviewed*,*'}),
     ('single', {'sloppytrailers': True}, {'addmysob': True}, 'sloppy', {}),
     ('with-cover', {}, {'addmysob': True}, 'defaults', {}),
-    ('with-cover', {}, {'covertrailers': True, 'addmysob': True}, 'covertrailers', {}),
+    ('with-cover', {}, {'addmysob': True, 'addlink': True}, 'addlink', {}),
     ('custody', {}, {'addmysob': True, 'copyccs': True}, 'unordered', {}),
     ('custody', {}, {'addmysob': True, 'copyccs': True}, 'ordered',
      {'trailer-order': 'Cc,Fixes*,Link*,Suggested*,Reviewed*,Tested*,*'}),
@@ -99,7 +100,10 @@ def test_parse_trailers(sampledir, source, expected):
      {'trailers-ignore-from': 'followup-reviewer1@example.com'}),
     ('partial-reroll', {}, {'addmysob': True}, 'defaults', {}),
     ('nore', {}, {}, 'defaults', {}),
+    ('non-git-patch', {}, {}, 'defaults', {}),
+    ('with-diffstat', {}, {}, 'defaults', {}),
     ('name-parens', {}, {}, 'defaults', {}),
+    ('bare-address', {}, {}, 'defaults', {}),
 ])
 def test_followup_trailers(sampledir, source, serargs, amargs, reference, b4cfg):
     b4.MAIN_CONFIG.update(b4cfg)
